﻿using PersonalFinance.Entity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonalFinance
{
    /// <summary>
    /// Service for communication to database
    /// </summary>
    public class ScheduledItemService
    {
        private EntityContext _context;
        public ScheduledItemService(EntityContext context)
        {
            _context = context;
        }

        /// <summary>
        /// Add ScheduledItem
        /// </summary>
        /// <param name="p">ScheduledItem</param>
        /// <returns>Added ScheduledItem</returns>
        public ScheduledPayment AddScheduledItem(ScheduledPayment p)
        {
            _context.ScheduledPayments.Add(p);
            _context.SaveChanges();
            return p;
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <returns>ScheduledItem</returns>
        public ScheduledPayment AddPayment(int price, PaymentCurrency currency, string purpose, PaymentGroup group, Frequency freq)
        {
            return AddPayment(price, currency, purpose, group, freq, DateTime.Now, null);
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <param name="lastUsed"></param>
        /// <returns>ScheduledItem</returns>
        public ScheduledPayment AddPayment(int price, PaymentCurrency currency, string purpose, PaymentGroup group, Frequency freq, DateTime lastUsed)
        {
            return AddPayment(price, currency, purpose, group, freq, lastUsed, null);
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <param name="lastUsed"></param>
        /// <param name="until">How long to be scheduled</param>
        /// <returns>ScheduledItem</returns>
        public ScheduledPayment AddPayment(int price, PaymentCurrency currency, string purpose, PaymentGroup group, Frequency freq, DateTime lastUsed, DateTime? until)
        {
            return AddScheduledItem(new ScheduledPayment()
            {
                Price = price,
                Currency = currency,
                Purpose = purpose,
                Group = group,
                Frequency = freq,
                LastUse = lastUsed,
                Until = until
            });
        }

        /// <summary>
        /// Retrieve all ScheduledItems
        /// </summary>
        /// <returns></returns>
        public List<ScheduledPayment> GetAllScheduledItems()
        {
            return _context.ScheduledPayments.ToList();
        }

        /// <summary>
        /// Get ScheduledItems by ID (primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>ScheduledItems</returns>
        public ScheduledPayment GetScheduledItem(int id)
        {
            return _context.ScheduledPayments.FirstOrDefault(i => i.ScheduledItemId == id);
        }

        /// <summary>
        /// Update ScheduledItems
        /// </summary>
        /// <param name="p">ScheduledItems to update</param>
        /// <returns>Updated ScheduledItems</returns>
        public ScheduledPayment UpdateScheduledItems(ScheduledPayment p)
        {
            _context.ScheduledPayments.Add(p);
            _context.SaveChanges();
            return p;
        }

        /// <summary>
        /// Delete ScheduledItems by ID  (Primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>Deleted ScheduledItems</returns>
        public ScheduledPayment DeleteScheduledItem(int id)
        {
            ScheduledPayment p = GetScheduledItem(id);
            if (p != null)
            {
                ScheduledPayment deleted = _context.ScheduledPayments.Remove(p);
                _context.SaveChanges();
                return deleted;
            }
            else
            {
                throw new IndexOutOfRangeException("ScheduledItem with ID " + id + " don't exist!");
            }
        }

        /// <summary>
        /// Delete ScheduledItems
        /// </summary>
        /// <param name="p">ScheduledItems to delete</param>
        /// <returns>Deleted ScheduledItems</returns>
        public ScheduledPayment DeleteScheduledItem(ScheduledPayment p)
        {
            ScheduledPayment deleted = _context.ScheduledPayments.Remove(p);
            _context.SaveChanges();
            return deleted;
        }
    }
}
